//差分法による Laplace方程式の解法
//矩形領域 {z|0<Re z<1, 0<Im z<1} において、
//厳密解 u(z)=(Re z)×(Im z) のLaplace方程式の境界値問題を解く
funcprot(0);
stacksize('max');

function w = u(z)
//u 厳密解
//z 引数
//w 関数値
w = real(z).*imag(z); // z が行列で与えられた場合も考慮
w = imag(z);
endfunction
function w = uu(j,k,h,l)
//uu 格子点上の厳密解
//j,k 格子点番号（jが実軸方向、kが虚軸方向）
//h,l 格子間隔（hが実軸方向、lが虚軸方向）
//w 関数値
w = u((j-1)*h+%i*(k-1)*l)
endfunction

//各パラメタの設定
m=30; H=1; h=H/m; //実軸方向の分割数、辺の長さ、格子間隔
n=30; L=1; l=L/n; //虚軸方向の分割数、辺の長さ、格子間隔
b=1/h^2; c=1/l^2; a=-2*(b+c); //5点差分パラメタ
A=eye((n+1)*(m+1),(n+1)*(m+1)); //係数行列の初期化（格子点数＝(m+1)×(n+1)に注意）
for j = 1:n-1
  for k = 1:m-1
    A(j+1+(n+1)*k,j+1+(n+1)*k) = a;
    A(j+1+(n+1)*k,j-1+1+(n+1)*k) = b;
    A(j+1+(n+1)*k,j+1+1+(n+1)*k) = b;
    A(j+1+(n+1)*k,j+1+(n+1)*(k-1)) = c;
    A(j+1+(n+1)*k,j+1+(n+1)*(k+1)) = c;
  end
end
B=zeros((n+1)*(m+1),1); //境界値に基づく右辺ベクトルの初期化
// left boundary
for j = 0:n
  k = 0;
// B(j+1+(n+1)*k) = j*(j-n)*4/(n*n);
B(j+1+(n+1)*k) = uu(j+1,k+1,h,l);
end
// right boundary
for j = 0:n
  k = m;
  B(j+1+(n+1)*k) = uu(j+1,k+1,h,l);
end
// bottom boundary
j = 0;
for k = 0:m
  B(j+1+(n+1)*k) = uu(j+1,k+1,h,l);
end
// top boundary
j = n;
for k = 0:m
  B(j+1+(n+1)*k) = uu(j+1,k+1,h,l);
end
// 直接法
F=A\B;
F=matrix(F,n+1,m+1);
grayplot(0:h:H,0:l:L,F);
pause;
// 反復法
// iterative meth.
D=diag(diag(A));
A=D-A;
F=B;
for i = 1:1000;
  F=D\(B+A*F);
  FF=matrix(F,n+1,m+1);
//途中経過を見るならコメントを外す
//  grayplot(0:h:H,0:l:L,FF);
end;
grayplot(0:h:H,0:l:L,FF);


